#!/python3
#- coding: utf-8 -*-
# for python3

# 2018.12.10 Version 0.2

# History
# 2018.12.7 Version 0.1

import os
import sys
import datetime
import configparser
import urllib.request
from io import BytesIO
from PIL import Image, ImageDraw, ImageFont
import re

MAX_TITLE_TEXT_LENGTH = 60

script_dir = os.path.dirname(__file__)
config_file_name = os.path.basename(__file__).replace(os.path.splitext(__file__)[1], ".conf")
config_path = os.path.join(script_dir, config_file_name)

config = configparser.ConfigParser()

config.read_string(open(config_path, "r", encoding="utf_8").read())

text_font = config["USER"]["text_font"]
text_size = int(config["USER"]["text_size"])
text_fill = eval(config["USER"]["text_fill"])
line_space = int(config["USER"]["line_space"])
line_alignment = int(config["USER"]["line_alignment"])
rectangle_alpha = int(config["USER"]["rectangle_alpha"])
rectangle_margin = int(config["USER"]["rectangle_margin"])
final_width = int(config["USER"]["final_width"])
final_height = int(config["USER"]["final_height"])
open_in = eval(config["USER"]["open_in"])
try:
	overflow_auto_shrink_rate = int(config["USER"]["overflow_auto_shrink_rate"])
except:
	overflow_auto_shrink_rate = 95

def to_int(n):
	return int(round(n))

def is_ios():
	return sys.platform == 'ios'

if (is_ios() == True):
	import console
	import photos
	import dialogs
	import ui
	import io

def to_alpha(n):
	return to_int(255 * n / 100)

def read_guess_jtextfile(file_name):
	encodings = ['iso2022_jp', 'iso2022_jp_1', 'iso2022_jp_2', 'iso2022_jp_3', 'iso2022_jp_ext',
	'euc_jp', 'euc_jis_2004', 'euc_jisx0213',
	'shift_jis', 'shift_jis_2004','shift_jisx0213',
	'utf_8', 'utf_16', 'latin_1', 'ascii']
	line_feeds = ["\r\n", "\r", "\n"]
	f = open(file_name, "rb")
	buf = f.read()
	f.close()
	line_feed = None
	text = None
	for enc in encodings:
		try:
			text = buf.decode(encoding=enc)
			break
		except:
			pass
	for le in line_feeds:
		if (text.find(le) != -1):
			break
		else:
			next
	if (isinstance(text, str)):
		return {"text":text, "encoding":enc, "line_feed":le}
	else:
		raise LookupError

# タイトルテキストを配列にスプリットしつつ空行を削除
def title_lines(txt):
	# 改行統一
	txt = txt.replace("\r\n", "\n").replace("\r", "\n")
	# 空行を削除
	return list(filter(lambda x: x.strip(), txt.split('\n')))

# タイトルテキスト / 画像入力
if (is_ios() == True):
	save_file = os.path.basename(__file__).replace(os.path.splitext(__file__)[1], ".txt")
	if (os.path.isfile(save_file)):
		with open(save_file, encoding='utf_8') as f:
			default_text = f.read()
	else:
		default_text = ""
	title_text = dialogs.text_dialog(title="タイトルテキストを入力", text=default_text, font=('HiraginoSans-W6', 24))
	if (isinstance(title_text, type(None))):
		exit()
	else:
		with open(save_file, mode='w', encoding='utf_8') as f:
			f.write(title_text)

	ast = photos.pick_asset()
	if (isinstance(ast, type(None))):
		exit()
	else:
		# https://forum.omz-software.com/topic/4599/heic-photo-file-issues-in-ios11
		pngdata = ui.Image.from_data(ast.get_image_data().getvalue()).to_png()
		im1 = Image.open(io.BytesIO(pngdata))
else:
	argvs = sys.argv
	argc = len(argvs)

	# argvs[0] スクリプト自身
	# argvs[1] テキストファイル
	# argvs[2] 画像ファイル or 画像へのURL

	if (argc < 3):
		print("引数の数が足りません %s <タイトルテキストファイル> <入力画像パスもしくはURL>" % os.path.basename(__file__))
		exit()
	else:
		if (not os.path.isfile(argvs[1])):
			print("指定のテキストファイルが存在しません: " + argvs[2])
			exit()
		else:
			title_text = read_guess_jtextfile(argvs[1])['text']
		# 引数がURLの場合は画像の直リンとして扱う
		if (re.match("^https?://", argvs[2]) != None):
			req = urllib.request.Request(argvs[2])
			with urllib.request.urlopen(req) as res:
				im1 = Image.open(BytesIO(res.read()))
			output_file_path = datetime.datetime.now().strftime("otg_%Y-%m-%d_%H-%M.jpg")
		else:
			# 入力ファイルの存在を確認
			if (not os.path.isfile(argvs[2])):
				print("指定の画像ファイルが存在しません: " + argvs[2])
				exit()
			else:
				im1 = Image.open(argvs[2])
				output_file_path = argvs[2].replace(os.path.splitext(argvs[2])[1], "_otg.jpg")

if (len(title_text) > MAX_TITLE_TEXT_LENGTH):
	print("タイトルテキストが長すぎませんか？: " + title_text)
	exit()

# 横幅は固定だから、入力された画像サイズに対し、等比の高さを出す
temp_height = to_int((final_width / im1.width) * im1.height)

# 横幅優先の縦横等比でresizeするためのサイズ
s = (int(final_width), int(temp_height))

# 等比のリサイズ実行
im1 = im1.resize(s, Image.LANCZOS)

# 仕上がりサイズでcropするためのサイズ
s = (final_width, final_height)

# 仕上がりサイズでのcrop実行
im1 = im1.crop((0, to_int((temp_height - final_height) / 2),
    final_width, to_int((temp_height - final_height) / 2) + final_height))

# 入力画像を合成のためにRGBA形式に変換
im1 = im1.convert("RGBA")

# cropした入力画像と同サイズの合成画像をRGBA形式で生成
im2 = Image.new("RGBA", s, (0, 0, 0, 0))

# 合成画像にオブジェクトを書き込むドロワー生成
drawer = ImageDraw.Draw(im2)

lines = title_lines(title_text)

line_count = len(lines)

font = ImageFont.truetype(text_font, text_size)

# 行幅が最長のものを調査
max_line_width = 0
for line in lines:
	# 描画される当該行の幅
	line_width = drawer.textsize(line, font=font)[0]
	if (line_width > max_line_width):
		max_line_width = line_width

# 設定text_sizeでは左右幅が溢れた場合に自動的に画像幅内に収める
if (max_line_width > final_width):
	scale_ratio = final_width / max_line_width
	text_size = to_int(text_size * scale_ratio * (overflow_auto_shrink_rate / 100))
	font = ImageFont.truetype(text_font, text_size)
	max_line_width = 0
	for line in lines:
		# 描画される当該行の幅
		line_width = drawer.textsize(line, font=font)[0]
		if (line_width > max_line_width):
			max_line_width = line_width

line_height = text_size + line_space

text_block_height = ((line_height * line_count) - line_space)

start_y_point = to_int((final_height - text_block_height) / 2)

rectangle_height = to_int(text_block_height * (1 + (rectangle_margin / 100)))

rectangle_offset = to_int((final_height / 2) - rectangle_height / 2)

# 黒いバックを書き込み
drawer.rectangle((0, rectangle_offset,
    final_width,
    (rectangle_height + rectangle_offset)),
    fill=(0, 0, 0, to_alpha(rectangle_alpha)))

line_index = 0

for line in lines:
	# https://tk0miya.hatenablog.com/entry/2014/07/12/005327
	# 描画される当該行の幅
	line_width = drawer.textsize(line, font=font)[0]
	if (line_alignment == 1):
		# センター合わせで行書き込み（画像中心から行幅の半分左にずらす）
		left_margin = to_int(final_width / 2 - line_width / 2)
	else:
		# 左寄せの場合は、最長の行がセンター合わせになるように調整
		left_margin = to_int((final_width - max_line_width) / 2)
	drawer.text((left_margin,
	    (start_y_point + line_height * line_index)),
	    line, font=font, fill=text_fill)
	line_index += 1

# [iOS]コンソールクリア
if (is_ios() == True):
	console.clear()

# 入力画像と合成画像を合成
im3 = Image.alpha_composite(im1, im2)

# RGBA → RGB変換でアルファ削除
im3 = im3.convert("RGB")

# 画像保存

# [iOS]画像表示
if (is_ios() == True):
	output_file_name = datetime.datetime.now().strftime("otg_%Y-%m-%d_%H-%M.jpg")
	im3.save(output_file_name)
	im3.show()
	if (open_in):
		console.open_in(output_file_name)
else:
	im3.save(output_file_path)
